@@ -29,14 +29,12 @@
 #include <dirent.h>
 #include <libgen.h>
 #include <ftw.h>
+#include <getopt.h>
+#include <locale.h>
+#include "config.h"
 #include "sequence.h"
 #include "parse.h"
-
-#include <getopt.h>
-
-#include <locale.h>
-#include <libintl.h>
-#define _(String) gettext (String)
+#include "misc.h"
 
 extern int
 do_set(
@@ -49,8 +47,8 @@ do_set(
 
 /* '-' stands for `process non-option arguments in loop' */
 #if !POSIXLY_CORRECT
-#  define CMD_LINE_OPTIONS "-:bknds:S:m:M:x:X:RLP"
-#  define CMD_LINE_SPEC "[-bkndRLP] { -s|-S|-m|-M|-x|-X ... } file ..."
+#  define CMD_LINE_OPTIONS "-:bkndm:M:x:X:RLP"
+#  define CMD_LINE_SPEC "[-bkndRLP] { -m|-M|-x|-X ... } file ..."
 #endif
 #define POSIXLY_CMD_LINE_OPTIONS "-:bkndm:M:x:X:"
 #define POSIXLY_CMD_LINE_SPEC "[-bknd] {-m|-M|-x|-X ... } file ..."
@@ -100,6 +98,16 @@ int chown_error;
 int promote_warning;
 
 
+static const char *xquote(const char *str)
+{
+	const char *q = quote(str);
+	if (q == NULL) {
+		fprintf(stderr, "%s: %s\n", progname, strerror(errno));
+		exit(1);
+	}
+	return q;
+}
+
 int
 has_any_of_type(
 	cmd_t cmd,
@@ -141,12 +149,13 @@ restore(
 		if (path_p == NULL) {
 			if (filename) {
 				fprintf(stderr, _("%s: %s: No filename found "
-				                  "in line %d, aborting\n"),
-					progname, filename, backup_line);
+						  "in line %d, aborting\n"),
+					progname, xquote(filename),
+					backup_line);
 			} else {
 				fprintf(stderr, _("%s: No filename found in "
-				                  "line %d of standard input, "
-					          "aborting\n"),
+						 "line %d of standard input, "
+						 "aborting\n"),
 					progname, backup_line);
 			}
 			goto getout;
@@ -166,14 +175,15 @@ restore(
 				     &line, NULL);
 		if (error != 0) {
 			fprintf(stderr, _("%s: %s: %s in line %d\n"),
-			        progname, filename, strerror(errno), line);
+			        progname, xquote(filename), strerror(errno),
+				line);
 			goto getout;
 		}
 
 		error = lstat(path_p, &stat);
 		if (opt_test && error != 0) {
-			fprintf(stderr, "%s: %s: %s\n", progname, path_p,
-			        strerror(errno));
+			fprintf(stderr, "%s: %s: %s\n", progname,
+				xquote(path_p), strerror(errno));
 			status = 1;
 		}
 		stat.st_uid = uid;
@@ -190,7 +200,8 @@ restore(
 			if (chown(path_p, uid, gid) != 0) {
 				fprintf(stderr, _("%s: %s: Cannot change "
 					          "owner/group: %s\n"),
-					progname, path_p, strerror(errno));
+					progname, xquote(path_p),
+					strerror(errno));
 				status = 1;
 			}
 		}
@@ -217,7 +228,8 @@ getout:
 	return status;
 
 fail:
-	fprintf(stderr, "%s: %s: %s\n", progname, filename, strerror(errno));
+	fprintf(stderr, "%s: %s: %s\n", progname, xquote(filename),
+		strerror(errno));
 	status = 1;
 	goto getout;
 }
@@ -230,13 +242,6 @@ void help(void)
 		progname, VERSION);
 	printf(_("Usage: %s %s\n"),
 		progname, cmd_line_spec);
-#if !POSIXLY_CORRECT
-	if (!posixly_correct) {
-		printf(_(
-"  -s, --set=acl           set the ACL of file(s), replacing the current ACL\n"
-"  -S, --set-file=file     read ACL entries to set from file\n"));
-	}
-#endif
 	printf(_(
 "  -m, --modify=acl        modify the current ACL(s) of file(s)\n"
 "  -M, --modify-file=file  read ACL entries to modify from file\n"
@@ -247,6 +252,8 @@ void help(void)
 #if !POSIXLY_CORRECT
 	if (!posixly_correct) {
 		printf(_(
+"      --set=acl           set the ACL of file(s), replacing the current ACL\n"
+"      --set-file=file     read ACL entries to set from file\n"
 "      --mask              do recalculate the effective rights mask\n"));
 	}
 #endif
@@ -291,6 +298,8 @@ static seq_t __seq;
 int __do_set(const char *file, const struct stat *stat,
              int flag, struct FTW *ftw)
 {
+	int saved_errno = errno;
+	
 	/* Process the target of a symbolic link, and traverse the link,
 	   only if doing a logical walk, or if the symbolic link was
 	   specified on the command line. Always skip symbolic links if
@@ -303,6 +312,13 @@ int __do_set(const char *file, const struct stat *stat,
 	if (do_set(file, stat, __seq))
 		__errors++;
 
+	if (flag == FTW_DNR && opt_recursive) {
+		/* Item is a directory which can't be read. */
+		fprintf(stderr, "%s: %s: %s\n",
+			progname, file, strerror(saved_errno));
+		return 0;
+	}
+
 	/* We also get here in non-recursive mode. In that case,
 	   return something != 0 to abort nftw. */
 
@@ -317,7 +333,8 @@ int walk_tree(const char *file, seq_t seq)
 	__errors = 0;
 	__seq = seq;
 	if (nftw(file, __do_set, 0, opt_walk_physical * FTW_PHYS) < 0) {
-		fprintf(stderr, "%s: %s\n", progname, strerror(errno));
+		fprintf(stderr, "%s: %s: %s\n", progname,
+			xquote(file), strerror(errno));
 		__errors++;
 	}
 	return __errors;
@@ -371,10 +388,11 @@ int main(int argc, char *argv[])
 	}
 #endif
 
+	setlocale(LC_CTYPE, "");
 	setlocale(LC_MESSAGES, "");
-	bindtextdomain(PKG_NAME, LOCALEDIR);
-	textdomain(PKG_NAME);
-	
+	bindtextdomain(PACKAGE, LOCALEDIR);
+	textdomain(PACKAGE);
+
 	while ((opt = getopt_long(argc, argv, cmd_line_options,
 		                  long_options, NULL)) != -1) {
 		/* we remember the two REMOVE_ACL commands of the set
@@ -471,11 +489,13 @@ int main(int argc, char *argv[])
 				                  seq_cmd, parse_mode) != 0) {
 					if (which < 0 ||
 					    (size_t) which >= strlen(optarg)) {
-						fprintf(stderr, _("%s: Option "
+						fprintf(stderr, _(
+							"%s: Option "
 						        "-%c incomplete\n"),
 							progname, opt);
 					} else {
-						fprintf(stderr, _("%s: Option "
+						fprintf(stderr, _(
+							"%s: Option "
 						        "-%c: %s near "
 							"character %d\n"),
 							progname, opt,
@@ -539,7 +559,8 @@ int main(int argc, char *argv[])
 					file = fopen(optarg, "r");
 					if (file == NULL) {
 						fprintf(stderr, "%s: %s: %s\n",
-							progname, optarg,
+							progname,
+							xquote(optarg),
 							strerror(errno));
 						status = 2;
 						goto cleanup;
@@ -559,15 +580,17 @@ int main(int argc, char *argv[])
 						errno = EINVAL;
 
 					if (file != stdin) {
-						fprintf(stderr, _("%s: %s in "
-						        "line %d of file %s\n"),
+						fprintf(stderr, _(
+							"%s: %s in line "
+						        "%d of file %s\n"),
 							progname,
 							strerror(errno),
 							lineno,
-							optarg);
+							xquote(optarg));
 					} else {
-						fprintf(stderr, _("%s: %s in "
-						        "line %d of standard "
+						fprintf(stderr, _(
+							"%s: %s in line "
+						        "%d of standard "
 							"input\n"), progname,
 							strerror(errno),
 							lineno);
@@ -595,7 +618,8 @@ int main(int argc, char *argv[])
 					file = fopen(optarg, "r");
 					if (file == NULL) {
 						fprintf(stderr, "%s: %s: %s\n",
-							progname, optarg,
+							progname,
+							xquote(optarg),
 							strerror(errno));
 						status = 2;
 						goto cleanup;
